package com.guwan.backend.jpaTest;

import com.guwan.backend.MergeStrategy.JoinBuilder;
import com.guwan.backend.MergeStrategy.JoinBuilder.FieldMapping;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class EmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;
    
    @PersistenceContext
    private EntityManager entityManager;

    /**
     * 获取员工详细信息，包含部门和项目信息
     * 使用字符串方式指定字段，并直接映射到DTO
     * 
     * @param departmentName 部门名称过滤条件
     * @param projectDeleted 项目是否已删除的过滤条件
     * @return 员工DTO列表
     */
    public List<EmployeeDTO> getEmployeeDetails(String departmentName, boolean projectDeleted) {
        // 创建查询构建器，从Employee实体开始
        JoinBuilder<Employee> builder = JoinBuilder.from(Employee.class);

        // 添加关联，使用字符串指定实体间的关系字段
        builder.join(Department.class, "departmentId", "id");
        builder.join(Project.class, "projectId", "id");

        // 添加查询条件，使用字符串指定字段
        if (departmentName != null && !departmentName.isEmpty()) {
            builder.equal(Department.class, "name", departmentName);
        }
        builder.equal(Project.class, "deleted", projectDeleted);


        // 使用selectAs设置字段别名，避免使用FieldMapping
        builder.selectAs("employeeId", Employee.class, "id");
        builder.selectAs("employeeName", Employee.class, "name");
        builder.selectAs("departmentName", Department.class, "name");
        builder.selectAs("projectName", Project.class, "name");

        // 直接执行查询并映射到DTO
        return builder.executeAndMap(entityManager, EmployeeDTO.class);
    }
    
    /**
     * 使用自定义条件查询员工信息
     * 使用selectAs来指定字段别名对应DTO构造函数参数
     * 
     * @param employeeName 员工名称过滤条件，可选
     * @param departmentName 部门名称过滤条件，可选
     * @return 员工DTO列表
     */
    public List<EmployeeDTO> findEmployees(String employeeName, String departmentName) {
        JoinBuilder<Employee> builder = JoinBuilder.from(Employee.class);
        
        // 添加关联，使用字符串
        builder.join(Department.class, "departmentId", "id");
        builder.join(Project.class, "projectId", "id");
        
        // 添加查询条件，使用字符串
        if (employeeName != null && !employeeName.isEmpty()) {
            builder.equal(Employee.class, "name", employeeName);
        }
        
        if (departmentName != null && !departmentName.isEmpty()) {
            builder.equal(Department.class, "name", departmentName);
        }
        
        // 方式2：使用selectAs直接指定字段别名
        builder.selectAs("employeeId", Employee.class, "id");
        builder.selectAs("employeeName", Employee.class, "name");
        builder.selectAs("departmentName", Department.class, "name");
        builder.selectAs("projectName", Project.class, "name");
        
        // 直接执行查询并映射到DTO
        return builder.executeAndMap(entityManager, EmployeeDTO.class);
    }
} 