package com.guwan.backend.jpaTest.mapper;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 查询结果映射工具类
 * 用于将原始查询结果转换为更有意义的对象
 */
public class QueryResultMapper {

    /**
     * 将Object[]结果列表映射为指定类型的对象列表
     * @param results 查询结果
     * @param mapper 映射函数
     * @return 映射后的对象列表
     */
    public static <T> List<T> mapToList(List<Object[]> results, Function<ResultRow, T> mapper) {
        return results.stream()
                .map(row -> mapper.apply(new ResultRow(row)))
                .collect(Collectors.toList());
    }
    
    /**
     * 将Object[]结果列表使用命名方式映射为指定类型的对象列表
     * @param results 查询结果
     * @param columnMap 列名到索引的映射
     * @param mapper 映射函数
     * @return 映射后的对象列表
     */
    public static <T> List<T> mapToListWithNames(List<Object[]> results, 
                                               Map<String, Integer> columnMap, 
                                               Function<NamedResultRow, T> mapper) {
        return results.stream()
                .map(row -> mapper.apply(new NamedResultRow(row, columnMap)))
                .collect(Collectors.toList());
    }
    
    /**
     * 结果行封装，提供类型安全的访问方法
     */
    public static class ResultRow {
        private final Object[] row;
        
        public ResultRow(Object[] row) {
            this.row = row;
        }
        
        public Long getLong(int index) {
            Object value = getValue(index);
            if (value == null) return null;
            if (value instanceof Long) return (Long) value;
            if (value instanceof Number) return ((Number) value).longValue();
            return Long.valueOf(value.toString());
        }
        
        public Integer getInteger(int index) {
            Object value = getValue(index);
            if (value == null) return null;
            if (value instanceof Integer) return (Integer) value;
            if (value instanceof Number) return ((Number) value).intValue();
            return Integer.valueOf(value.toString());
        }
        
        public String getString(int index) {
            Object value = getValue(index);
            return value == null ? null : value.toString();
        }
        
        public Boolean getBoolean(int index) {
            Object value = getValue(index);
            if (value == null) return null;
            if (value instanceof Boolean) return (Boolean) value;
            if (value instanceof Number) return ((Number) value).intValue() != 0;
            return Boolean.valueOf(value.toString());
        }
        
        public Object getValue(int index) {
            if (index < 0 || index >= row.length) {
                throw new IndexOutOfBoundsException("索引超出范围: " + index);
            }
            return row[index];
        }
    }
    
    /**
     * 命名结果行封装，通过列名访问数据
     */
    public static class NamedResultRow {
        private final Object[] row;
        private final Map<String, Integer> columnMap;
        
        public NamedResultRow(Object[] row, Map<String, Integer> columnMap) {
            this.row = row;
            this.columnMap = columnMap;
        }
        
        public Long getLong(String columnName) {
            Object value = getValue(columnName);
            if (value == null) return null;
            if (value instanceof Long) return (Long) value;
            if (value instanceof Number) return ((Number) value).longValue();
            return Long.valueOf(value.toString());
        }
        
        public Integer getInteger(String columnName) {
            Object value = getValue(columnName);
            if (value == null) return null;
            if (value instanceof Integer) return (Integer) value;
            if (value instanceof Number) return ((Number) value).intValue();
            return Integer.valueOf(value.toString());
        }
        
        public String getString(String columnName) {
            Object value = getValue(columnName);
            return value == null ? null : value.toString();
        }
        
        public Boolean getBoolean(String columnName) {
            Object value = getValue(columnName);
            if (value == null) return null;
            if (value instanceof Boolean) return (Boolean) value;
            if (value instanceof Number) return ((Number) value).intValue() != 0;
            return Boolean.valueOf(value.toString());
        }
        
        public Object getValue(String columnName) {
            Integer index = columnMap.get(columnName);
            if (index == null) {
                throw new IllegalArgumentException("列名不存在: " + columnName);
            }
            if (index < 0 || index >= row.length) {
                throw new IndexOutOfBoundsException("索引超出范围: " + index);
            }
            return row[index];
        }
    }
}